/*
 * T13_HID.c
 *
 *  Created on: 31-03-2013
 *       Autor: fredek
 *
 *       CPU: ATtiny13A
 *       FUSEBIT: LOW=0x39 HIGH=0xEB
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

// definicje opnie
#define WAIT_5M 525 	// przerwa ponownego odpalenia hid ~5 min.
#define WAIT_15M 1575	// minimalny czas pracy hid ~15 min.
#define WAIT_KEY 20		// blokada odczytu klawiszy (1 sek)
#define WORK_TIME 50400	// max czas pracy do autowyczenia ~8 godz.

// definicje podczenia portw
#define LED_PIN  (1<<PB0)	// dioda LED (anoda do 5V)
#define KEY1_PIN (1<<PB1)	// klawisz 1
#define KEY2_PIN (1<<PB2)	// klawisz 2
#define HID1_PIN (1<<PB3)	// HID nr 1
#define HID2_PIN (1<<PB4)	// HID nr 2

// definicje klawiatury
#define KEY_PIN ( KEY1_PIN | KEY2_PIN )
#define KEY_S1 !( PINB & KEY1_PIN ) // klawisz S1 wcinity
#define KEY_S2 !( PINB & KEY2_PIN ) // klawisz S2 wcinity

// definicje dla LED
#define LED_ON PORTB &= ~LED_PIN
#define LED_OFF PORTB |= LED_PIN
#define LED_TOG PORTB ^= LED_PIN

// definicje dla 2-ch relay module
#define HID_PIN ( HID1_PIN | HID2_PIN )
#define HID1_ON PORTB &= ~HID1_PIN
#define HID1_OFF PORTB |= HID1_PIN
#define HID2_ON PORTB &= ~HID2_PIN
#define HID2_OFF PORTB |= HID2_PIN

// definicje typw
typedef uint8_t u08;
typedef uint16_t u16;
typedef uint32_t u32;

// definicje zmiennych
volatile u16 lock1=WAIT_5M;	// czas blokady dla HID 1
volatile u16 lock2=WAIT_5M; // czas blokady dla HID 2
volatile u08 cnt1=WAIT_KEY;	// blokada odczytu klawisza S1
volatile u08 cnt2=WAIT_KEY;	// blokada odczytu klawisza S1
volatile u32 time1, time2;	// czas do autowyczenia
volatile u08 sw1, sw2;		// sygnalizacja nacisniecia klawisza

int main(void)
{

	DDRB |= LED_PIN | HID_PIN ;		// ustawiamy wyjscia
	LED_OFF;						// wyczamy LED
	HID1_OFF;						// wyczamy HID1
	HID2_OFF;						// wyczamy HID2
	DDRB &= ~KEY_PIN;				// kierunek pinw klawiszy - wejciowy
	PORTB |= KEY_PIN;				// podcignicie pinw klawiszy do VCC

	// Ustawienie przerwania watchdog
	WDTCR |= (1<<WDTIE);			// Watchdog Timer Interrupt Enable
	WDTCR |= (1<<WDP2)|(1<<WDP0);	// 0.5 s

	// Ustawienie Timer0 CTC mode
	TCCR0A |= (1<<WGM01);			// tryb CTC
	TCCR0B |= (1<<CS02)|(1<<CS00);	// preskaler = 1024
	OCR0A = 234;					// ~ 20 Hz dla zegara 4,8 MHz
	TIMSK0 |= (1<<OCIE0A);			// Timer/Counter0 Output Compare Match

	sei();				// wczenie przerwa

	// definiujemy stan pocztkowy hid-w
	u08 hid1=0;
	u08 hid2=0;

	while (1)	// gwna ptla
	{
		// ===== obsuga HID nr 1 =====
		if (!lock1) 	// brak blokady
		{
			if (sw1) 		// nacisnieto klawisz
			{
				if (hid1) 		// hid wczony
				{
					HID1_OFF;			// wyczamy
					hid1=0;
					lock1=WAIT_5M;		// blokada 5 min na ostygnicie
				}
				else			// hid wyczony
				{
					HID1_ON;			// wczamy
					hid1=1;
					lock1=WAIT_15M;		// blokada 15 min na rozgrzanie
					time1=WORK_TIME;	// ustawienie czasu autowyczenia
				}
				sw1=0;		// kasujemy stan klawisza
			}
		}

		// ===== obsuga HID nr 2 =====
		if (!lock2) 	// brak blokady
		{
			if (sw2) 		// nacisnieto klawisz
			{
				if (hid2) 		// hid wczony
				{
					HID2_OFF;			// wyczamy
					hid2=0;
					lock2=WAIT_5M;		// blokada 5 min na ostygnicie
				}
				else			// hid wyczony
				{
					HID2_ON;			// wczamy
					hid2=1;
					lock2=WAIT_15M;		// blokada 15 min na rozgrzanie
					time2=WORK_TIME;	// ustawienie czasu autowyczenia
				}
				sw2=0;		// kasujemy stan klawisza
			}
		}

		// obsuga automatycznego wyczenia oswietlenia
		if (hid1 && !time1)		// obsuga automatycznego wyczenia HID1
		{
			HID1_OFF;			// wyczamy
			hid1=0;
			lock1=WAIT_5M;		// blokada 5 min na ostygnicie
		}
		if (hid2 && !time2)		// obsuga automatycznego wyczenia HID2
		{
			HID2_OFF;			// wyczamy
			hid2=0;
			lock2=WAIT_5M;		// blokada 5 min na ostygnicie
		}

		// wyczamy WDT na czas obsugi klawiszy (eliminacja gaszenia LED)
		if (cnt1 || cnt2) WDTCR &= ~(1<<WDTIE); else WDTCR |= (1<<WDTIE);
	}
}

// ===== obsuga blokady czasowej (przerwanie 2 Hz) =====
ISR (WDT_vect)
{
	// zmniejszanie czasu blokady
	if (lock1) lock1--;
	if (lock2) lock2--;
	if (time1) time1--;
	if (time2) time2--;

	// jeeli jakas operacja bdzie si wykonywaa po upyniciu
	// czasu blokady to migamy dugo LED
	if ( (lock1 && sw1) || (lock2 && sw2) ) LED_TOG; else LED_OFF;

	// jeeli nie ma adnej operacji do wykonania ale aktywna
	// jest jakas blokada to krtkie mignicie LED
	if ( (lock1 || lock2) && !sw1  && !sw2 )
		{
		LED_ON;
		_delay_ms(2);
		LED_OFF;
		}
}

// ===== obsuga klawiatury (przerwanie 20 Hz) =====
ISR (TIM0_COMPA_vect)
{
	/* ===== obsuga awaryjnego zmianu stanu =====
	 * nastpuje po naciniciu przycisku S1 lub S2
	 * przez minimum 5 sek (gasnie LED) */
	static u08 reset_lock1 = 0;
	static u08 reset_lock2 = 0;

	if ( KEY_S1 ) reset_lock1++; else reset_lock1=0;
	if ( KEY_S2 ) reset_lock2++; else reset_lock2=0;

	if (reset_lock1>=100)	// min. 5 sek
	{
		lock1=0;		// kasowanie blokady
		sw1=1;			// klawisz wcisnity - nastpi zmiana stanu
		LED_OFF;		// sygnalizacja upywu 5 sek
		while (KEY_S1);	// czekamy na zwolnienie przycisku
		cnt1=WAIT_KEY;	// blokada odczytu
	}
	if (reset_lock2>=100)	// min. 5 sek
	{
		lock2=0;		// kasowanie blokady
		sw2=1;			// klawisz wcisnity - nastpi zmiana stanu
		LED_OFF;		// sygnalizacja upywu 5 sek
		while (KEY_S2);	// czekamy na zwolnienie przycisku
		cnt2=WAIT_KEY;	// blokada odczytu
	}

	// ===== obsuga S1 =====
	if ( !(cnt1) ) 		// jeeli cnt = 0 - brak blokady S1, sprawdzamy
	{
		if ( KEY_S1 )
		{
			sw1=!sw1;		// zmiana stanu klawisza
			cnt1=WAIT_KEY;	// blokada odczytu klawiszy
		}
	}
	else
		{
			cnt1--;		// blokada klawiszy - zmniejszamy tylko licznik
			LED_ON;
		}

	// ===== obsuga S2 =====
	if ( !(cnt2) ) 		// jeeli cnt = 0 - brak blokady S2, sprawdzamy
	{
		if ( KEY_S2 )
		{
			sw2=!sw2;		// zmiana stanu klawisza
			cnt2=WAIT_KEY;	// blokada odczytu klawiszy
		}
	}
	else
		{
			cnt2--;		// blokada klawiszy - zmniejszamy tylko licznik
			LED_ON;
		}

}
